/*
 * Created on 16-Jan-2005
 */
package net.sourceforge.uml2svg;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Attribute;
import org.jdom.Comment;
import org.jdom.Content;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.Namespace;
import org.jdom.Text;
import org.jdom.filter.Filter;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * @author donald
 */
public class Concatenator {
	public Namespace xslNamespace = Namespace.getNamespace("xsl", "http://www.w3.org/1999/XSL/Transform");
	public Namespace umlNamespace = Namespace.getNamespace("UML", "org.omg.xmi.namespace.UML");
	public Namespace uml2Namespace = Namespace.getNamespace("UML2", "org.omg.xmi.namespace.UML2");
	public Namespace svgNamespace = Namespace.getNamespace("", "http://www.w3.org/2000/svg");
	private SAXBuilder builder;
	private XMLOutputter outputter;
	private Document resultDoc = new Document();
	private Element resultRoot = new Element("stylesheet", xslNamespace);
	private Set fileNames = new HashSet();

	public Concatenator(String startFileName) throws IOException, JDOMException {
	    builder = new SAXBuilder();
		builder.setIgnoringElementContentWhitespace(false);
		builder.setExpandEntities(false);
		
		outputter = new XMLOutputter();
		outputter.setFormat(Format.getRawFormat().setEncoding("UTF-8"));

		resultDoc.setRootElement(resultRoot);
		resultRoot.addNamespaceDeclaration(umlNamespace);
		resultRoot.addNamespaceDeclaration(uml2Namespace);
		resultRoot.addNamespaceDeclaration(svgNamespace);
		resultRoot.setAttribute(new Attribute("version", "1.0"));
		resultRoot.setAttribute(new Attribute("exclude-result-prefixes", "UML UML2"));
		
		Element outputElement = new Element("output", xslNamespace);

		resultRoot.addContent(new Text("\n"));
		outputElement.setAttribute("media-type", "image/svg+xml");
		outputElement.setAttribute("method", "xml");
		outputElement.setAttribute("indent", "yes");
		resultRoot.addContent(outputElement);
		resultRoot.addContent(new Text("\n"));
		
		add(startFileName);
	}
	
	
	public void add(String path) throws IOException, JDOMException {
		int pos = path.lastIndexOf("/");
		File file;
		String fileName, dirName;
		if (pos>0) {
			fileName = path.substring(pos+1);
			dirName = path.substring(0, pos);
			file = new File(dirName, fileName);
		} else {
			fileName = path;
			dirName = "";
			file = new File(path);
		}
		
		if (!fileNames.contains(file.getCanonicalFile())) {
			fileNames.add(file.getCanonicalFile());

			Document doc = builder.build(file);
			Element root = doc.getRootElement();
			List contentList = root.getContent();
			
			// imports
			while (!contentList.isEmpty()) {
				Content content = (Content)contentList.remove(0);
				if (content instanceof Element &&
						((Element)content).getName().equals("import") &&
						((Element)content).getNamespace().equals(xslNamespace)) {
					Element importElement = (Element)content;
					String href = importElement.getAttributeValue("href");
					if (dirName.length()>0) {
						href = dirName+"/"+href; 
					}
					add(href);
				} else {
					if (!(content instanceof Text) &&
							!(content instanceof Comment)) {
						contentList.add(0, content);
						break;
					}
				}
			}
			
			resultRoot.addContent(new Text("\n"));
			resultRoot.addContent(new Comment("**********************************************************************"));			
			resultRoot.addContent(new Text("\n"));
			
			
			// comments
			List commentList = doc.getContent(new Filter() {
				public boolean matches(java.lang.Object obj) {
					return obj instanceof Comment;
				}
			});
			while (!commentList.isEmpty()) {
				Comment comment = (Comment)commentList.remove(0);
				doc.removeContent(comment);
				resultRoot.addContent(comment);
			}

			// content
			while (!contentList.isEmpty()) {
				Content content = (Content)contentList.remove(0);
				if (content instanceof Element &&
					((Element)content).getName().equals("output") &&
					((Element)content).getNamespace().equals(xslNamespace)) {
					// ignore <output> tags
				} else {
					root.removeContent(content);
					resultRoot.addContent(content);
				}
			}

		}	
	}
	
	public void output(String fileName) throws IOException {
		File file = new File(fileName);
		Writer writer = new BufferedWriter(new FileWriter(file));
		outputter.output(resultDoc, writer);
		writer.flush();
	}
	
	public String toString() {
		return outputter.outputString(resultDoc);
	}

	public static void main(String[] args) throws Exception {
		if (args.length != 2) {
			System.err.println("<starting_xsl_file> <concatenated_xsl_file>");
		}
		Concatenator concat = new Concatenator(args[0]);
		concat.output(args[1]);
	}
}
