<?php
session_start();
class DiagramChooser{
  var $fileName;
  var $tmpFileName;
  var $resultFileName;
  var $prefixPath;

  function DiagramChooser(){
    $this->prefixPath = "";
    if(!$this->moveFile()){
      return;
    }
    if(extension_loaded('xsl')){
      $count = $this->xslCountDiagrams();
    }
    else if(extension_loaded('xslt')){
      $count = $this->xsltCountDiagrams();
    }
    else if(extension_loaded('domxml')){
      $count = $this->domxmlCountDiagrams();
    } else {
    	$this->displayHTML('no xsl, xslt or domxml mods installed on this server.', false);
        return;
    }
    if($count === FALSE){
      return;
    }
    switch($count){
      case -1:
        return;
        break;
      case 0:
        $this->displayHTML('<p class="err">this file does not have any Diagram Interchange valid information.</p>', false);
        return;
      case 1:
        $message = "<p>one diagram detected. press <b>Transform</b> to continue.</p>\n"; 
        $message .= "<div class=\"form\">\n<form action=\"transform.php\" method=\"post\">\n<div>\n";
        $message .= "<input type=\"hidden\" name=\"locxmifile\" value=\"".$this->fileName."\"/>\n";
        $message .= "<input id=\"submitButton\" class=\"button_\" type=\"submit\" value=\"Transform\" />\n";
        $message .= "</div>\n</form>\n</div>\n";
        $this->displayHTML($message, true);
        break;
      default:
        $message = "<p>Which diagrams do you want to transform into SVG?</p>\n"; 
        $message .= "<div class=\"form\">\n<form action=\"transform.php\" method=\"post\">\n<div>\n";
        $message .= "<input type=\"hidden\" name=\"locxmifile\" value=\"".$this->fileName."\"/>\n";
        if(extension_loaded('xsl')){
          $DiagramList = $this->xslListDiagrams();
        }
        else if(extension_loaded('xslt')){
          $DiagramList = $this->xsltListDiagrams();
        }
        else if(extension_loaded('domxml')){
          $DiagramList = $this->domxmlListDiagrams();
        }
        if(!$DiagramList){
          return;
        }
        $message .= $DiagramList;
        $message .= "<br/><input class=\"button_\" type=\"submit\" value=\"Transform\" />\n";
        $message .= "</div>\n</form>\n</div>\n";
        $this->displayHTML($message, false);
        break;
    }
  }

  function xslCountDiagrams(){
    $StyleSheet = new DOMDocument;
    if(!$StyleSheet->load($this->prefixPath."xsl/DiagramCount.xsl")){
      $this->displayHTML("<p class='err'>stylesheet not found</p>", false);
      return false;
    };
    $File = new DOMDocument;
    if(!$File->load($this->prefixPath.$this->resultFileName)){
      $this->displayHTML("<p class='err'>file not found</p>", false);
      return false;
    };
    $XslProcessor = new XSLTProcessor();
    $XslProcessor->importStyleSheet($StyleSheet);
    $TransformResult = $XslProcessor->transformToXML($File);
    if ($TransformResult === FALSE)
    {
      $this->displayHTML("<p class='err'>unknown XSLTProcessor Error</p>", false);
      return -1;
    }
    return $TransformResult;
  }

  function domxmlCountDiagrams(){
    $StyleSheet = domxml_xslt_stylesheet_file($this->prefixPath."xsl/DiagramCount.xsl");
    if(!$StyleSheet){
      $this->displayHTML("<p class='err'>stylesheet not found</p>", false);
      return false;
    };
    $File = @domxml_open_file($this->prefixPath.$this->resultFileName);
    if(!$File){
      $this->displayHTML("<p class='err'>invalid file</p>", false);
      return false;
    };
    $TransformResult = $StyleSheet->process($File);
    if (!$TransformResult)
    {
      $this->displayHTML("<p class='err'>unknown DomXsltStylesheet Error</p>", false);
      return -1;
    }
    return $StyleSheet->result_dump_mem($TransformResult);
  }

  function xsltCountDiagrams(){
    $XSLTProcessor = xslt_create();
    $TransformResult = @xslt_process($XSLTProcessor, $this->prefixPath.$this->resultFileName, $this->prefixPath."xsl/DiagramCount.xsl");
    if ($TransformResult === FALSE){
      $this->displayHTML("<p class='err'>XSLT ERROR: " . xslt_error($XSLTProcessor) . " (" . xslt_errno($XSLTProcessor) . ")</p>", false);
      return -1;
    }
    xslt_free($XSLTProcessor);
    return $TransformResult;
  }

  function xslListDiagrams(){
    $StyleSheet = new DOMDocument;
    if(!$StyleSheet->load($this->prefixPath."xsl/DiagramList.xsl")){
      $this->displayHTML("<p class='err'>stylesheet not found</p>", false);
      return false;
    };
    $File = new DOMDocument;
    if(!$File->load($this->prefixPath.$this->resultFileName)){
      $this->displayHTML("<p class='err'>file not found</p>", false);
      return false;
    };
    $XslProcessor = new XSLTProcessor();
    $XslProcessor->importStyleSheet($StyleSheet);
    $TransformResult = $XslProcessor->transformToXML($File);
    if ($TransformResult === FALSE)
    {
      $this->displayHTML("<p class='err'>unknown XSLTProcessor Error</p>", false);
      return false;
    }
    return $TransformResult;
  }

  function domxmlListDiagrams(){
    $StyleSheet = domxml_xslt_stylesheet_file($this->prefixPath."xsl/DiagramList.xsl");
    if(!$StyleSheet){
      $this->displayHTML("<p class='err'>stylesheet not found</p>", false);
      return false;
    };
    $File = domxml_open_file($this->prefixPath.$this->resultFileName);
    if(!$File){
      $this->displayHTML("<p class='err'>invalid file</p>", false);
      return false;
    };
    $TransformResult = $StyleSheet->process($File);
    if (!$TransformResult)
    {
      $this->displayHTML("<p class='err'>unknown DomXsltStylesheet Error</p>", false);
      return false;
    }
    return $StyleSheet->result_dump_mem($TransformResult);
  }
  
  function xsltListDiagrams(){
    $XSLTProcessor = xslt_create();
    $TransformResult = @xslt_process($XSLTProcessor, $this->prefixPath.$this->resultFileName, $this->prefixPath."xsl/DiagramList.xsl");
    if (!$TransformResult){
      $this->displayHTML("<p class='err'>XSLT ERROR: " . xslt_error($XSLTProcessor) . " (" . xslt_errno($XSLTProcessor) . ")</p>", false);
      return false;
    }
    xslt_free($XSLTProcessor);
    return $TransformResult;
  }

  function moveFile(){
    if(!isset($_FILES['xmifile']['tmp_name'])){
      $this->displayHTML('<p class="err">please <a href="demo.php">specify</a> a file to transform.</p>', false);
      return false;
    }
    if(!$_FILES['xmifile']['tmp_name']){
      $this->displayHTML('<p class="err">upload failed. sorry.</p>', false);
      return false;
    }

    $this->tmpFileName = $_FILES['xmifile']['tmp_name'];
    $this->fileName = $_FILES['xmifile']['name'];
    $this->resultFileName = "tmp/".session_id().$this->fileName;

    if (@move_uploaded_file($this->tmpFileName, $this->resultFileName) === FALSE){
      $this->displayHTML('<p class="err">move failed. sorry.</p>', false);
      return false;
    }
    return true;
  }

  function displayHTML($message, $autoclick){
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
  <head>
    <title>uml2svg online transforming tool</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-2" />
    <link href="skin.css" type="text/css" rel="stylesheet" />
  </head>
  <body <?php if(isset($autoclick) && ($autoclick == true)) echo("onload=\"document.getElementById('submitButton').click();\""); ?>>
    <h1 class="header">uml2svg online transforming tool</h1>
<?php
    echo($message);
?>
  </body>
</html>
<?php
  }
}

new DiagramChooser();
?>