<?php
session_start();
ini_set('memory_limit', '64M');
ini_set('max_input_time', '600');
set_time_limit(0);

class TransformToSVG{
  var $fileName;
  var $inputFileName;
  var $outputFileName;
  var $selectedDiagram;
  var $prefixPath;

  function TransformToSVG(){
    $this->prefixPath = "";

    if(isset($_POST['locxmifile'])){
      $this->fileName = $_POST['locxmifile'];
      $this->inputFileName = "tmp/".session_id().$this->fileName;
      $this->outputFileName = $this->inputFileName.".svg";
      if(isset($_POST['DiagramID'])){
        $this->selectedDiagram = $_POST['DiagramID'];
      }
      if(extension_loaded('xsl')){
        if(!$this->xslTransform()){
          return;
        }
      }
      else if(extension_loaded('xslt')){
        if(!$this->xsltTransform()){
          return;
        }
      }
      else if(extension_loaded('domxml')){
        if(!$this->domxmlTransform()){
          return;
        }
      }
      $this->OutputRedirectHeader();
    }
  }

  function xslTransform(){
    $StyleSheet = new DOMDocument;
    if(!$StyleSheet->load($this->prefixPath."xsl/uml2svg.xsl")){
      $this->displayHTML("<p class='err'>stylesheet not found</p>");
      return false;
    };
    $File = new DOMDocument;
    if(!$File->load($this->prefixPath.$this->inputFileName)){
      $this->displayHTML("<p class='err'>file not found</p>");
      return false;
    };

    $XslProcessor = new XSLTProcessor();
    $XslProcessor->importStyleSheet($StyleSheet);
    if(isset($this->selectedDiagram) && ($this->selectedDiagram != "ALL_DIAGRAMS")){
      $XslProcessor->setParameter("", "SelectedDiagramId", $this->selectedDiagram);
    }
    $TransformResult = $XslProcessor->transformToURI($File, $this->prefixPath.$this->outputFileName);
    if ($TransformResult === FALSE)
    {
      $this->displayHTML("<p class='err'>unknown XSLTProcessor Error</p>");
      return false;
    }
    return true;
  }

  function domxmlTransform(){
    $StyleSheet = domxml_xslt_stylesheet_file($this->prefixPath."xsl/uml2svg.xsl");
    if(!$StyleSheet){
      $this->displayHTML("<p class='err'>stylesheet not found</p>");
      return false;
    };
    $File = domxml_open_file($this->prefixPath.$this->inputFileName);
    if(!$File){
      $this->displayHTML("<p class='err'>file not found</p>");
      return false;
    };

    if(!isset($this->selectedDiagram) || ($this->selectedDiagram == "ALL_DIAGRAMS")){
      $TransformResult = $StyleSheet->process($File);
    }
    else{
      $TransformResult = $StyleSheet->process($File, array("SelectedDiagramId" => $_POST['DiagramID']));
    }
    if (!$TransformResult)
    {
      $this->displayHTML("<p class='err'>unknown DomXmlStylesheet Error</p>");
      return false;
    }
    $StyleSheet->result_dump_file($TransformResult, $this->prefixPath.$this->outputFileName);
    return true;
  }

  function xsltTransform(){
    $XSLTProcessor = xslt_create();
    if(!isset($this->selectedDiagram) || ($this->selectedDiagram == "ALL_DIAGRAMS")){
      $TransformResult = @xslt_process($XSLTProcessor, $this->prefixPath.$this->inputFileName, $this->prefixPath."xsl/uml2svg.xsl", $this->prefixPath.$this->outputFileName);
    }
    else{
      $TransformResult = @xslt_process($XSLTProcessor, $this->prefixPath.$this->inputFileName, $this->prefixPath."xsl/uml2svg.xsl", $this->prefixPath.$this->outputFileName, NULL, 
        array("SelectedDiagramId" => $_POST['DiagramID']));
    }
    if (!$TransformResult)
    {
      $this->displayHTML("<p class='err'>XSLT ERROR: " . xslt_error($XSLTProcessor) . " (" . xslt_errno($XSLTProcessor) . ")</p>");
      return false;
    }
    xslt_free($XSLTProcessor);
    return true;
  }

  function displayHTML($message){
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
  <head>
    <title>uml2svg online transforming tool</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-2" />
    <link href="skin.css" type="text/css" rel="stylesheet" />
  </head>
  <body>
    <h1 class="header">uml2svg online transforming tool</h1>
<?php
    echo($message);
?>
  </body>
</html>
<?php
  }

  function OutputSVGHeader(){
    header("Content-type: image/svg+xml");
  }

  function OutputRedirectHeader(){
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
 "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html>
  <head>
    <title>uml2svg online transforming tool</title>
    <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-2" />
    <link href="skin.css" type="text/css" rel="stylesheet" />
    <meta http-equiv="refresh" content="0;URL=<?php echo($this->outputFileName); ?>"/>
  </head>
  <body>
    <h1 class="header">uml2svg online transforming tool</h1>
    <p>redirecting to <a href="<?php echo($this->outputFileName); ?>">the SVG file</a>...</p>
  </body>
</html>
<?php
  }
}

new TransformToSVG();
?>